##############################################################################
#
# Zope Public License (ZPL) Version 0.9.4
# ---------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that the following
# conditions are met:
# 
# 1. Redistributions in source code must retain the above
#    copyright notice, this list of conditions, and the following
#    disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions, and the following
#    disclaimer in the documentation and/or other materials
#    provided with the distribution.
# 
# 3. Any use, including use of the Zope software to operate a
#    website, must either comply with the terms described below
#    under "Attribution" or alternatively secure a separate
#    license from Digital Creations.
# 
# 4. All advertising materials, documentation, or technical papers
#    mentioning features derived from or use of this software must
#    display the following acknowledgement:
# 
#      "This product includes software developed by Digital
#      Creations for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
# 5. Names associated with Zope or Digital Creations must not be
#    used to endorse or promote products derived from this
#    software without prior written permission from Digital
#    Creations.
# 
# 6. Redistributions of any form whatsoever must retain the
#    following acknowledgment:
# 
#      "This product includes software developed by Digital
#      Creations for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
# 7. Modifications are encouraged but must be packaged separately
#    as patches to official Zope releases.  Distributions that do
#    not clearly separate the patches from the original work must
#    be clearly labeled as unofficial distributions.
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND
#   ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
#   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT
#   SHALL DIGITAL CREATIONS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
#   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
#   IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
#   THE POSSIBILITY OF SUCH DAMAGE.
# 
# Attribution
# 
#   Individuals or organizations using this software as a web site
#   must provide attribution by placing the accompanying "button"
#   and a link to the accompanying "credits page" on the website's
#   main entry point.  In cases where this placement of
#   attribution is not feasible, a separate arrangment must be
#   concluded with Digital Creations.  Those using the software
#   for purposes other than web sites must provide a corresponding
#   attribution in locations that include a copyright using a
#   manner best suited to the application environment.
# 
# This software consists of contributions made by Digital
# Creations and many individuals on behalf of Digital Creations.
# Specific attributions are listed in the accompanying credits
# file.
# 
##############################################################################
# (C) Copyright 2000,2002
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrea Mennucc <mennucc1@debian.org>
# derived from etcAuthSource by Andrew Milton <akm@theinternet.com.au>
#  and from smbAuthSource by  Andrew Milton <akm@theinternet.com.au>

"""

Authenticates off of an imap server

"""

import os, string, Acquisition

from Globals import HTMLFile, MessageDialog, INSTANCE_HOME
from string import join,strip,split,lower,upper,find

from OFS.Folder import Folder

import zLOG

from check_imap import check_imap

try:
	from crypt import crypt
except:
	from Products.exUserFolder.fcrypt.fcrypt import crypt

from Products.exUserFolder.exUserFolder import exUserFolder
from Products.exUserFolder.Plugins import PluginRegister

path_split=os.path.split
path_join=os.path.join
exists=os.path.exists

def manage_addimapAuthSource(self, REQUEST):

	""" """
	imap_server=REQUEST['imapauth_server']
	default_role=REQUEST['imapauth_default_role']
	ob=imapAuthSource(imap_server=imap_server, default_role=default_role)
	self._setObject('imapAuthSource', ob, None, None, 0)
	self.currentAuthSource=ob	

manage_addimapAuthSourceForm=HTMLFile('manage_addimapAuthSourceForm', globals())
manage_editimapAuthSourceForm=HTMLFile('manage_editimapAuthSourceForm', globals())

class imapAuthSource(Folder):
	""" """

	meta_type='Authorisation Source'
	id		 ='imapAuthSource'
	title	 ='imap System Authorisation'
	icon ='misc_/exUserFolder/exUserFolderPlugin.gif'

	manage_properties=HTMLFile('properties', globals())

	manage_editForm=manage_editimapAuthSourceForm
	manage_tabs=Acquisition.Acquired

	#
	# You can define this to go off and do the authentication instead of
	# using the basic one inside the User Object
	#
	def remoteAuthMethod(self, username, password):
		'Authenticate a username/password combination against the imap server'
		if username == '' or username == None :
			zLOG.LOG('imapAuthSource',
				 zLOG.DEBUG,
				 'got null username; auth failed' )
			return 0
		try:
			status=check_imap(user=username,site=self.imap_server,
					   passwd=password)
			zLOG.LOG('imapAuthSource',
				 zLOG.TRACE,
				 'checked for %s\%s status %s' %                				 (self.imap_server, username,str(status)))
			return 7 == status
		except Exception, e:
			import sys
			zLOG.LOG('imapAuthSource',
				 zLOG.BLATHER,
				 'SessionError (%s) for %s\%s (usually means login failure); auth failed' % (e, self.imap_server, username),
				 '\n',
				 sys.exc_info() )
			return 0
		
	authenticate=remoteAuthMethod


	def __init__(self, default_role, imap_server):
		self.imap_server=imap_server
		self.default_role=default_role

	def manage_editAuthSource(self, REQUEST):

		""" """
		self.imap_server=REQUEST['imapauth_server']
		self.default_role=REQUEST['imapauth_default_role']

	def cryptPassword(self, username, password):
		u=self.listOneUser(username)
		if not u:
			salt = username[:2]
		else:
			salt = u[0]['password'][:2]
		secret = crypt(password, salt)
		return secret

	#
	# We don't let you delete, create, or edit users
	#
	def deleteUsers(self, userids):
		pass

	def createUser(self, username, password, roles):
		pass

	def updateUser(self, username, password, roles):
		self.currentPropSource.setUserProperty(username=username,
						       key='_roles',
						       value=roles)
	def listUserNames(self):
		return []
		
	def listUsers(self):
		return []

	def getUsers(self):
		return []
	
	def listOneUser(self, username):
		roles=[]
		try:
			if self.currentPropSource:
				roles=self.currentPropSource.getUserProperty(username=username, key='_roles', default=[])
		except Exception, msg:
			zLOG.LOG('imapAuthSource',
				 zLOG.DEBUG,
				 "listOneUser cant find roles for username: '%s'}, reason %s" % (username,str(msg) ))
		
		if not roles:
			roles=[] # make sure it's a list...

		#why on earth?? username = string.lower(username)

		zLOG.LOG('imapAuthSource',
			 zLOG.TRACE,
			 "listOneUser returning {username: '%s', password: '', roles: %s}" % (username, roles)
			)
		return [{'username':username, 'password':'', 'roles':roles},]

		
imapAuthReg=PluginRegister('imapAuthSource', 'imap Based Authentication Source',
						  imapAuthSource, manage_addimapAuthSourceForm,
						  manage_addimapAuthSource,
						  manage_editimapAuthSourceForm)
exUserFolder.authSources['imapAuthSource']=imapAuthReg


